==> org-lsst-ccs-subsystem-mcm-shuttersim-1.0-sources.jar: org/lsst/ccs/subsystem/mcm/shuttersim/ShutterSim.java <==
package org.lsst.ccs.subsystem.mcm.shuttersim;

import java.time.Duration;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.framework.Module;
import org.lsst.ccs.utilities.logging.Logger;

public class ShutterSim extends Module {

	private ShutterInternalState state = ShutterInternalState.CLOSED_OFF;

	private final ScheduledThreadPoolExecutor scheduler = new ScheduledThreadPoolExecutor(4);

	protected static final Logger log = Logger.getLogger("org.lsst.ccs.subsystem.mcm");

	public ShutterSim(String name, int tickMillis) {
		super(name, tickMillis);
	}

	@Override
	public void initModule() {
		getSubsystem().updateAgentState("", state.getPublicState(), state.getPublicReadinessState());
	}

	@Command
	public void prepare() {
		log.info("ShutterSim command: prepare");
		setState(state.prepare(this));
	}

	@Command
	public void expose(long millis) {
		log.info("ShutterSim command: expose " + millis);
		setState(state.expose(this, Duration.ofMillis(millis)));
	}

	public long getLastEffectiveExposureTime() {
		return state.getLastEffectiveExposureTime();
	}

	public long getLastTotalOpeningTime() {
		return state.getLastTotalOpeningTime();
	}

	public void setState(ShutterInternalState s) {
		if (state == s)
			return;

		state.exit(this);
		state = s;
		state.enter(this);

		getSubsystem().updateAgentState("", state.getPublicState(), state.getPublicReadinessState());

		log.info("ShutterSim state: " + state.getPublicState() + " " + state.getPublicReadinessState());

	}

	public ScheduledFuture<ShutterInternalState> scheduleTransition(Duration delay, ShutterInternalState to) {
		log.debug(" scheduling to " + to + " delay " + delay);
		return scheduler.schedule(() -> ShutterInternalState.scheduled(this, state, to), delay.toMillis(),
				TimeUnit.MILLISECONDS);
	}

	Logger getLogger() {
		return log;
	}

	@Override
	public void tick() {
		// empty
	}

}

==> org-lsst-ccs-subsystem-mcm-shuttersim-2.0-sources.jar: org/lsst/ccs/subsystem/mcm/shuttersim/ShutterSim.java <==
package org.lsst.ccs.subsystem.mcm.shuttersim;

import java.time.Duration;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.lsst.ccs.UsesSubsystem;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.framework.Module;
import org.lsst.ccs.utilities.logging.Logger;

public class ShutterSim extends Module implements UsesSubsystem {

	private ShutterInternalState state = ShutterInternalState.CLOSED_OFF;

	private final ScheduledThreadPoolExecutor scheduler = new ScheduledThreadPoolExecutor(4);

	protected static final Logger log = Logger.getLogger("org.lsst.ccs.subsystem.mcm");

	@Override
	public void initModule() {
		getSubsystem().updateAgentState(state.getPublicState(), state.getPublicReadinessState());
	}

	@Command
	public void prepare() {
		log.info("ShutterSim command: prepare");
		setState(state.prepare(this));
	}

	@Command
	public void expose(long millis) {
		log.info("ShutterSim command: expose " + millis);
		setState(state.expose(this, Duration.ofMillis(millis)));
	}

	@Command
	public void close() {
		log.info("ShutterSim command: close");
		setState(state.close(this));
	}

	public long getLastEffectiveExposureTime() {
		return state.getLastEffectiveExposureTime();
	}

	public long getLastTotalOpeningTime() {
		return state.getLastTotalOpeningTime();
	}

	public void setState(ShutterInternalState s) {
		if (state == s)
			return;

		state.exit(this);
		state = s;
		state.enter(this);

		getSubsystem().updateAgentState(state.getPublicState(), state.getPublicReadinessState());

		log.info("ShutterSim state: " + state.getPublicState() + " " + state.getPublicReadinessState());

	}

	public ShutterInternalState getState() {
		return state;
	}

	public ScheduledFuture<ShutterInternalState> scheduleTransition(Duration delay, ShutterInternalState from,
			ShutterInternalState to) {
		log.info(" scheduling to " + to + " delay " + delay + " from " + from);
		return scheduler.schedule(() -> ShutterInternalState.scheduled(this, from, to), delay.toMillis(),
				TimeUnit.MILLISECONDS);
	}

	public void clearSchedule() {
		for (Runnable r : scheduler.getQueue()) {
			if (r instanceof ScheduledFuture<?>) {
				((ScheduledFuture) r).cancel(false);
			}
		}
	}

	Logger getLogger() {
		return log;
	}

	@Override
	public void tick() {
		// empty
	}

}
